
/*
 * \file IMP/base/base_config.h
 * \brief Provide macros to mark functions and classes as exported
 *        from a DLL/.so, and to set up namespaces
 *
 * When building the module, IMPBASE_EXPORTS should be defined, and when
 * using the module externally, it should not be. Classes and functions
 * declared in the module's headers should then be marked with
 * IMPBASEEXPORT if they are intended to be part of the API and
 * they are not defined entirely in a header.
 *
 * The Windows build environment requires applications to mark exports in
 * this way; we use the same markings to set the visibility of ELF symbols
 * if we have compiler support.
 *
 * All code in this module should live in the IMP::base namespace.
 * This is simply achieved by wrapping things with the
 * IMPBASE_BEGIN_NAMESPACE and IMPBASE_END_NAMESPACE macros.
 * There are similar macros for module code that is designed to be for
 * internal use only.
 *
 * This header is auto-generated by tools/build/setup_module.py;
 * it should not be edited manually.
 *
 * Copyright 2007-2013 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPBASE_CONFIG_H
#define IMPBASE_CONFIG_H

#include <IMP/base/base_config.h>
#include <string>


#ifdef _MSC_VER

#ifdef IMPBASE_EXPORTS
#define IMPBASEEXPORT __declspec(dllexport)
#else // EXPORTS
#define IMPBASEEXPORT __declspec(dllimport)
#endif // EXPORTS

#else // _MSC_VER

#ifdef GCC_VISIBILITY
#define IMPBASEEXPORT __attribute__ ((visibility("default")))
#else // GCC_VISIBILITY
#define IMPBASEEXPORT
#endif // GCC_VISIBILITY
#endif // _MSC_VER

#if defined(_MSC_VER) && !defined(SWIG)
#ifdef IMPBASE_EXPORTS

#define IMPBASE_EXPORT_TEMPLATE(name)                 template class __declspec(dllexport) name

#else //EXPORTS

#define IMPBASE_EXPORT_TEMPLATE(name)                 template class __declspec(dllimport) name

#endif // EXPORTS

#else // MSC and SWIG
#define IMPBASE_EXPORT_TEMPLATE(name) IMP_REQUIRE_SEMICOLON_NAMESPACE

#endif // MSC and SWIG

#if !defined(SWIG) && !defined(IMP_DOXYGEN)

#define IMPBASE_BEGIN_NAMESPACE \
IMP_COMPILER_ENABLE_WARNINGS \
namespace IMP { namespace base { \

#define IMPBASE_END_NAMESPACE \
} } \
IMP_COMPILER_DISABLE_WARNINGS

#define IMPBASE_BEGIN_INTERNAL_NAMESPACE \
IMPBASE_BEGIN_NAMESPACE namespace internal {


#define IMPBASE_END_INTERNAL_NAMESPACE \
} IMPBASE_END_NAMESPACE

#else
#define IMPBASE_BEGIN_NAMESPACE \
namespace IMP { namespace base {

#define IMPBASE_END_NAMESPACE \
} }

#define IMPBASE_BEGIN_INTERNAL_NAMESPACE \
IMPBASE_BEGIN_NAMESPACE namespace internal {


#define IMPBASE_END_INTERNAL_NAMESPACE \
} IMPBASE_END_NAMESPACE

#endif

#define IMP_BUILD IMP_RELEASE
#define IMP_HAS_DEPRECATED 1
#define IMP_HAS_LOG IMP_VERBOSE
#define IMP_HAS_CHECKS IMP_USAGE
#define IMP_DEBUG 0
#define IMP_RELEASE 1
#define IMP_SILENT 0
#define IMP_PROGRESS 2
#define IMP_TERSE 3
#define IMP_VERBOSE 4
#define IMP_NONE 0
#define IMP_USAGE 1
#define IMP_INTERNAL 2
#define IMP_COMPILER_HAS_AUTO 0
#define IMP_COMPILER_HAS_DEBUG_VECTOR 1
#define IMP_COMPILER_HAS_NULLPTR 0
#define IMP_BASE_NO_BOOST_RANDOM
#define IMP_BASE_HAS_BOOST_RANDOM 0
#define IMP_BASE_NO_GPERFTOOLS
#define IMP_BASE_HAS_GPERFTOOLS 0
#define IMP_BASE_NO_LOG4CXX
#define IMP_BASE_HAS_LOG4CXX 0
#define IMP_BASE_NO_TCMALLOC_HEAPCHECKER
#define IMP_BASE_HAS_TCMALLOC_HEAPCHECKER 0
#define IMP_BASE_NO_TCMALLOC_HEAPPROFILER
#define IMP_BASE_HAS_TCMALLOC_HEAPPROFILER 0

//  functions are defined explicitly for swig

namespace IMP { namespace base {
/** \name Standard module methods
  All \imp modules have a set of standard methods to help get information
  about the module and about files associated with the module.
  @{
  */
#if !defined(SWIG)
IMPBASEEXPORT std::string get_module_version();
#endif

#if !defined(SWIG)
// swig will whine about duplicate definitions of function
inline std::string get_module_name() {
   return "IMP::base";
}
#endif

} } //namespace



#if !defined(SWIG)

namespace IMP { namespace base {

//! Return the full path to installed data
/** Each module has its own data directory, so be sure to use
    the version of this function in the correct module. To read
    the data file "data_library" that was placed in the \c data
    directory of module "mymodule", do something like
    \code
    std::ifstream in(IMP::mymodule::get_data_path("data_library"));
    \endcode
    This will ensure that the code works when \imp is installed or
    used via the \c tools/imppy.sh script.
*/
IMPBASEEXPORT std::string get_data_path(std::string file_name);

//! Return the path to installed example data for this module
/** Each module has its own example directory, so be sure to use
    the version of this function in the correct module.  For example
    to read the file \c example_protein.pdb located in the
    \c examples directory of the IMP::atom module, do
    \code
    IMP::atom::read_pdb(IMP::atom::get_example_path("example_protein.pdb", model));
    \endcode
    This will ensure that the code works when \imp is installed or
    used via the \c tools/imppy.sh script.
*/
IMPBASEEXPORT std::string get_example_path(std::string file_name);
/** @} */


} } // namespace

#endif // SWIG

#include <IMP/base/compiler_macros.h>

#ifdef IMP_DOXYGEN
/** \namespace IMP::base
    \brief See \ref IMP_base_overview "IMP.base Overview" for more information.
    */
#endif

#endif  /* IMPBASE_CONFIG_H */
