/**
 *  \file IMP/core/QuadConstraint.h
 *  \brief Use a QuadModifier applied to a ParticleQuadsTemp to
 *  maintain an invariant
 *
 *  This file is generated by a script (core/tools/make-container).
 *  Do not edit directly.
 *
 *  Copyright 2007-2013 IMP Inventors. All rights reserved.
 */

#ifndef IMPCORE_QUAD_CONSTRAINT_H
#define IMPCORE_QUAD_CONSTRAINT_H

#include <IMP/core/core_config.h>
#include <IMP/internal/TupleConstraint.h>
#include <IMP/QuadModifier.h>
#include <IMP/QuadDerivativeModifier.h>
#include <IMP/score_state_macros.h>

IMPCORE_BEGIN_NAMESPACE
//! Apply a QuadFunction to a Quad
/** The score state is passed up to two QuadModifiers, one to
    apply before evaluation and the other after. The one after
    should take a DerivativeAccumulator as its last argument for
    QuadModifier::apply() and will only be called if
    the score was computed with derivatives.

    \see container::QuadsConstraint
 */
class QuadConstraint :
#if defined(IMP_DOXYGEN) || defined(SWIG)
public Constraint
#else
public IMP::kernel::internal::TupleConstraint<QuadModifier,
                                      QuadDerivativeModifier>
#endif
{
public:
  /** before and after are the modifiers to apply before and after
      evaluate.
   */
  QuadConstraint(QuadModifier *before,
                      QuadDerivativeModifier *after,
                      const ParticleQuad& vt,
                      std::string name="QuadConstraint %1%"):
      IMP::kernel::internal::TupleConstraint<QuadModifier,
                                      QuadDerivativeModifier>
      (before, after, vt, name)
      {
  }

#if defined(IMP_DOXYGEN) || defined(SWIG)
  protected:
  void do_update_attributes();
  void do_update_derivatives(DerivativeAccumulator *da);
  virtual ModelObjectsTemp do_get_inputs() const;
  virtual ModelObjectsTemp do_get_outputs() const;
  IMP_OBJECT_METHODS(QuadConstraint);
#endif
};


IMPCORE_END_NAMESPACE

#endif  /* IMPCORE_QUAD_CONSTRAINT_H */
