
/*
 * \file IMP/modeller/modeller_config.h
 * \brief Provide macros to mark functions and classes as exported
 *        from a DLL/.so, and to set up namespaces
 *
 * When building the module, IMPMODELLER_EXPORTS should be defined, and when
 * using the module externally, it should not be. Classes and functions
 * declared in the module's headers should then be marked with
 * IMPMODELLEREXPORT if they are intended to be part of the API and
 * they are not defined entirely in a header.
 *
 * The Windows build environment requires applications to mark exports in
 * this way; we use the same markings to set the visibility of ELF symbols
 * if we have compiler support.
 *
 * All code in this module should live in the IMP::modeller namespace.
 * This is simply achieved by wrapping things with the
 * IMPMODELLER_BEGIN_NAMESPACE and IMPMODELLER_END_NAMESPACE macros.
 * There are similar macros for module code that is designed to be for
 * internal use only.
 *
 * This header is auto-generated by tools/build/setup_module.py;
 * it should not be edited manually.
 *
 * Copyright 2007-2013 IMP Inventors. All rights reserved.
 *
 */

#ifndef IMPMODELLER_CONFIG_H
#define IMPMODELLER_CONFIG_H

#include <IMP/base/base_config.h>
#include <string>


#ifdef _MSC_VER

#ifdef IMPMODELLER_EXPORTS
#define IMPMODELLEREXPORT __declspec(dllexport)
#else // EXPORTS
#define IMPMODELLEREXPORT __declspec(dllimport)
#endif // EXPORTS

#else // _MSC_VER

#ifdef GCC_VISIBILITY
#define IMPMODELLEREXPORT __attribute__ ((visibility("default")))
#else // GCC_VISIBILITY
#define IMPMODELLEREXPORT
#endif // GCC_VISIBILITY
#endif // _MSC_VER

#if defined(_MSC_VER) && !defined(SWIG)
#ifdef IMPMODELLER_EXPORTS

#define IMPMODELLER_EXPORT_TEMPLATE(name)                 template class __declspec(dllexport) name

#else //EXPORTS

#define IMPMODELLER_EXPORT_TEMPLATE(name)                 template class __declspec(dllimport) name

#endif // EXPORTS

#else // MSC and SWIG
#define IMPMODELLER_EXPORT_TEMPLATE(name) IMP_REQUIRE_SEMICOLON_NAMESPACE

#endif // MSC and SWIG

#if !defined(SWIG) && !defined(IMP_DOXYGEN)

#define IMPMODELLER_BEGIN_NAMESPACE \
IMP_COMPILER_ENABLE_WARNINGS \
namespace IMP { namespace modeller { \

#define IMPMODELLER_END_NAMESPACE \
} } \
IMP_COMPILER_DISABLE_WARNINGS

#define IMPMODELLER_BEGIN_INTERNAL_NAMESPACE \
IMPMODELLER_BEGIN_NAMESPACE namespace internal {


#define IMPMODELLER_END_INTERNAL_NAMESPACE \
} IMPMODELLER_END_NAMESPACE

#else
#define IMPMODELLER_BEGIN_NAMESPACE \
namespace IMP { namespace modeller {

#define IMPMODELLER_END_NAMESPACE \
} }

#define IMPMODELLER_BEGIN_INTERNAL_NAMESPACE \
IMPMODELLER_BEGIN_NAMESPACE namespace internal {


#define IMPMODELLER_END_INTERNAL_NAMESPACE \
} IMPMODELLER_END_NAMESPACE

#endif

#define IMP_MODELLER_USE_IMP_ALGEBRA
#define IMP_MODELLER_HAS_IMP_ALGEBRA 1
#define IMP_MODELLER_USE_IMP_BASE
#define IMP_MODELLER_HAS_IMP_BASE 1
#define IMP_MODELLER_USE_IMP_DISPLAY
#define IMP_MODELLER_HAS_IMP_DISPLAY 1
#define IMP_MODELLER_USE_IMP_KERNEL
#define IMP_MODELLER_HAS_IMP_KERNEL 1
#define IMP_MODELLER_USE_IMP_SCORE_FUNCTOR
#define IMP_MODELLER_HAS_IMP_SCORE_FUNCTOR 1
#define IMP_MODELLER_USE_BOOST_FILESYSTEM
#define IMP_MODELLER_HAS_BOOST_FILESYSTEM 1
#define IMP_MODELLER_USE_BOOST_PROGRAMOPTIONS
#define IMP_MODELLER_HAS_BOOST_PROGRAMOPTIONS 1
#define IMP_MODELLER_USE_BOOST_SYSTEM
#define IMP_MODELLER_HAS_BOOST_SYSTEM 1

//  functions are defined explicitly for swig

namespace IMP { namespace modeller {
/** \name Standard module methods
  All \imp modules have a set of standard methods to help get information
  about the module and about files associated with the module.
  @{
  */
#if !defined(SWIG)
IMPMODELLEREXPORT std::string get_module_version();
#endif

#if !defined(SWIG)
// swig will whine about duplicate definitions of function
inline std::string get_module_name() {
   return "IMP::modeller";
}
#endif

} } //namespace

#if !defined(IMP_DOXYGEN) && !defined(SWIG)

#include <IMP/base/Showable.h>
#include <IMP/base/hash.h>

namespace IMP { namespace modeller {
using ::IMP::base::Showable;
using ::IMP::base::operator<<;
using ::IMP::base::hash_value;
} } // namespace
namespace IMP { namespace modeller { namespace internal {
using ::IMP::base::Showable;
using ::IMP::base::operator<<;
using ::IMP::base::hash_value;
} } } // namespace

#endif // !defined(SWIG) && !defined(IMP_DOXYGEN)


#if !defined(SWIG)

namespace IMP { namespace modeller {

//! Return the full path to installed data
/** Each module has its own data directory, so be sure to use
    the version of this function in the correct module. To read
    the data file "data_library" that was placed in the \c data
    directory of module "mymodule", do something like
    \code
    std::ifstream in(IMP::mymodule::get_data_path("data_library"));
    \endcode
    This will ensure that the code works when \imp is installed or
    used via the \c tools/imppy.sh script.
*/
IMPMODELLEREXPORT std::string get_data_path(std::string file_name);

//! Return the path to installed example data for this module
/** Each module has its own example directory, so be sure to use
    the version of this function in the correct module.  For example
    to read the file \c example_protein.pdb located in the
    \c examples directory of the IMP::atom module, do
    \code
    IMP::atom::read_pdb(IMP::atom::get_example_path("example_protein.pdb", model));
    \endcode
    This will ensure that the code works when \imp is installed or
    used via the \c tools/imppy.sh script.
*/
IMPMODELLEREXPORT std::string get_example_path(std::string file_name);
/** @} */


} } // namespace

#endif // SWIG

#include <IMP/base/compiler_macros.h>

#ifdef IMP_DOXYGEN
/** \namespace IMP::modeller
    \brief See \ref IMP_modeller_overview "IMP.modeller Overview" for more information.
    */
#endif

#endif  /* IMPMODELLER_CONFIG_H */
