/**
 *  \file RMF/publication_decorators.h
 *  \brief Helper functions for manipulating RMF files.
 *
 *  Copyright 2007-2013 IMP Inventors. All rights reserved.
 *
 */

#ifndef RMF_PUBLICATION_DECORATORS_H
#define RMF_PUBLICATION_DECORATORS_H

#include <RMF/config.h>
#include <RMF/infrastructure_macros.h>
#include <RMF/NodeHandle.h>
#include <RMF/FileHandle.h>
#include <RMF/Decorator.h>
#include <RMF/Factory.h>
#include <RMF/constants.h>
#include <RMF/internal/utility.h>
#include <RMF/internal/paths.h>
#include <boost/array.hpp>
#include <boost/lexical_cast.hpp>

RMF_ENABLE_WARNINGS
namespace RMF {

/** Information regarding a publication.

       See also JournalArticle and JournalArticleConstFactory.
     */
    class JournalArticleConst:
        public Decorator<NodeConstHandle> {
    friend class JournalArticleConstFactory;
    std::string get_name() const {return "JournalArticle";}
    private:
    typedef Decorator<NodeConstHandle> P;
    StringKey title_;
StringKey journal_;
StringKey pubmed_id_;
IntKey year_;
StringsKey authors_;
    JournalArticleConst(NodeConstHandle nh,
                  StringKey title,
StringKey journal,
StringKey pubmed_id,
IntKey year,
StringsKey authors):
       P(nh),
title_(title),
journal_(journal),
pubmed_id_(pubmed_id),
year_(year),
authors_(authors) {
    ;
    }
    public:
    /** The article title. */
String get_title() const {
  try{
  return P::get_value(title_);
  } RMF_DECORATOR_CATCH( );
}
/** The article title. */
Strings get_all_titles() const {
  try {
  return P::get_all_values(title_);
  } RMF_DECORATOR_CATCH( );
}
/** The journal title. */
String get_journal() const {
  try{
  return P::get_value(journal_);
  } RMF_DECORATOR_CATCH( );
}
/** The journal title. */
Strings get_all_journals() const {
  try {
  return P::get_all_values(journal_);
  } RMF_DECORATOR_CATCH( );
}
/** The pubmed ID. */
String get_pubmed_id() const {
  try{
  return P::get_value(pubmed_id_);
  } RMF_DECORATOR_CATCH( );
}
/** The pubmed ID. */
Strings get_all_pubmed_ids() const {
  try {
  return P::get_all_values(pubmed_id_);
  } RMF_DECORATOR_CATCH( );
}
/** The publication year. */
Int get_year() const {
  try{
  return P::get_value(year_);
  } RMF_DECORATOR_CATCH( );
}
/** The publication year. */
Ints get_all_years() const {
  try {
  return P::get_all_values(year_);
  } RMF_DECORATOR_CATCH( );
}
/** A list of authors as Lastname, Firstname */
Strings get_authors() const {
  try{
  return P::get_value(authors_);
  } RMF_DECORATOR_CATCH( );
}
/** A list of authors as Lastname, Firstname */
StringsList get_all_authorss() const {
  try {
  return P::get_all_values(authors_);
  } RMF_DECORATOR_CATCH( );
}
    static std::string get_decorator_type_name() {
         return "JournalArticleConst";
    }
    };

    typedef std::vector<JournalArticleConst>
            JournalArticleConsts;

/** Information regarding a publication.

       See also JournalArticleConst and JournalArticleFactory.
     */
    class JournalArticle:
        public Decorator<NodeHandle> {
    friend class JournalArticleFactory;
    std::string get_name() const {return "JournalArticle";}
    private:
    typedef Decorator<NodeHandle> P;
    StringKey title_;
StringKey journal_;
StringKey pubmed_id_;
IntKey year_;
StringsKey authors_;
    JournalArticle(NodeHandle nh,
                  StringKey title,
StringKey journal,
StringKey pubmed_id,
IntKey year,
StringsKey authors):
       P(nh),
title_(title),
journal_(journal),
pubmed_id_(pubmed_id),
year_(year),
authors_(authors) {
    ;
    }
    public:
    /** The article title. */
String get_title() const {
  try{
  return P::get_value(title_);
  } RMF_DECORATOR_CATCH( );
}
/** The article title. */
Strings get_all_titles() const {
  try {
  return P::get_all_values(title_);
  } RMF_DECORATOR_CATCH( );
}
/** The article title. */
void set_title(String v) {
  try {
  P::set_value(title_,
            v);
  } RMF_DECORATOR_CATCH( );
}
/** The journal title. */
String get_journal() const {
  try{
  return P::get_value(journal_);
  } RMF_DECORATOR_CATCH( );
}
/** The journal title. */
Strings get_all_journals() const {
  try {
  return P::get_all_values(journal_);
  } RMF_DECORATOR_CATCH( );
}
/** The journal title. */
void set_journal(String v) {
  try {
  P::set_value(journal_,
            v);
  } RMF_DECORATOR_CATCH( );
}
/** The pubmed ID. */
String get_pubmed_id() const {
  try{
  return P::get_value(pubmed_id_);
  } RMF_DECORATOR_CATCH( );
}
/** The pubmed ID. */
Strings get_all_pubmed_ids() const {
  try {
  return P::get_all_values(pubmed_id_);
  } RMF_DECORATOR_CATCH( );
}
/** The pubmed ID. */
void set_pubmed_id(String v) {
  try {
  P::set_value(pubmed_id_,
            v);
  } RMF_DECORATOR_CATCH( );
}
/** The publication year. */
Int get_year() const {
  try{
  return P::get_value(year_);
  } RMF_DECORATOR_CATCH( );
}
/** The publication year. */
Ints get_all_years() const {
  try {
  return P::get_all_values(year_);
  } RMF_DECORATOR_CATCH( );
}
/** The publication year. */
void set_year(Int v) {
  try {
  P::set_value(year_,
            v);
  } RMF_DECORATOR_CATCH( );
}
/** A list of authors as Lastname, Firstname */
Strings get_authors() const {
  try{
  return P::get_value(authors_);
  } RMF_DECORATOR_CATCH( );
}
/** A list of authors as Lastname, Firstname */
StringsList get_all_authorss() const {
  try {
  return P::get_all_values(authors_);
  } RMF_DECORATOR_CATCH( );
}
/** A list of authors as Lastname, Firstname */
void set_authors(Strings v) {
  try {
  P::set_value(authors_,
            v);
  } RMF_DECORATOR_CATCH( );
}
    static std::string get_decorator_type_name() {
         return "JournalArticle";
    }
    };

    typedef std::vector<JournalArticle>
            JournalArticles;

/** Create decorators of type JournalArticle.

       See also JournalArticle and JournalArticleConstFactory.
    */
    class JournalArticleFactory:
       public Factory<FileHandle>
                 {
    private:
    typedef Factory<FileHandle> P;
    StringKey title_;
StringKey journal_;
StringKey pubmed_id_;
IntKey year_;
StringsKey authors_;
    public:
    JournalArticleFactory(FileHandle fh):
      P(){
    Category cat = fh.get_category("publication");
RMF_UNUSED(cat);
title_=fh.get_key<StringTraits>(cat,
        "title");
journal_=fh.get_key<StringTraits>(cat,
        "journal");
pubmed_id_=fh.get_key<StringTraits>(cat,
        "pubmed id");
year_=fh.get_key<IntTraits>(cat,
        "year");
authors_=fh.get_key<StringsTraits>(cat,
        "authors");;
    }
    /** Get a JournalArticle for nh.*/
    JournalArticle get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::ORGANIZATIONAL), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  JournalArticle");;
      return JournalArticle(nh, title_,
journal_,
pubmed_id_,
year_,
authors_);
    }
    /** Check whether nh has all the attributes required to be a
        JournalArticle.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::ORGANIZATIONAL)
    && nh.get_has_value(title_)
    && nh.get_has_value(journal_)
    && nh.get_has_value(pubmed_id_)
    && nh.get_has_value(year_)
    && nh.get_has_value(authors_);
    }
    };

    typedef std::vector<JournalArticleFactory>
            JournalArticleFactories;

/** Create decorators of type JournalArticle.

       See also JournalArticleConst and JournalArticleFactory.
    */
    class JournalArticleConstFactory:
       public Factory<FileConstHandle>
                 {
    private:
    typedef Factory<FileConstHandle> P;
    StringKey title_;
StringKey journal_;
StringKey pubmed_id_;
IntKey year_;
StringsKey authors_;
    public:
    JournalArticleConstFactory(FileConstHandle fh):
      P(){
    Category cat = fh.get_category("publication");
RMF_UNUSED(cat);
title_=fh.get_key<StringTraits>(cat,
        "title");
journal_=fh.get_key<StringTraits>(cat,
        "journal");
pubmed_id_=fh.get_key<StringTraits>(cat,
        "pubmed id");
year_=fh.get_key<IntTraits>(cat,
        "year");
authors_=fh.get_key<StringsTraits>(cat,
        "authors");;
    }
    /** Get a JournalArticleConst for nh.*/
    JournalArticleConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::ORGANIZATIONAL), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  JournalArticle");;
      return JournalArticleConst(nh, title_,
journal_,
pubmed_id_,
year_,
authors_);
    }
    /** Check whether nh has all the attributes required to be a
        JournalArticleConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::ORGANIZATIONAL)
    && nh.get_has_value(title_)
    && nh.get_has_value(journal_)
    && nh.get_has_value(pubmed_id_)
    && nh.get_has_value(year_)
    && nh.get_has_value(authors_);
    }
    };

    typedef std::vector<JournalArticleConstFactory>
            JournalArticleConstFactories;

} /* namespace RMF */
RMF_DISABLE_WARNINGS

#endif /* RMF_PUBLICATION_DECORATORS_H */
